/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2009 Embedded Artists AB
 *
 * Description:
 *    Hardware interface to the display
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lcd_hw.h"
#include "lpc313x_timer_driver.h"

/******************************************************************************
 * Defines, macros, and typedefs
 *****************************************************************************/

#define LCD_COMMAND_16  (*((volatile unsigned short *) 0x20000000)) 
#define LCD_DATA_16     (*((volatile unsigned short *) 0x20010000))

#define EA_DISP_QVGA_LCD_V2_ID  0x8989
#define EA_DISP_QVGA_OLED_V1_ID 0x63D6

#define delayMs(ms) timer_wait_ms(NULL, ms);

/******************************************************************************
 * Local variables
 *****************************************************************************/

static UNS_8 activeDisplay = 0;

/******************************************************************************
 * Local functions
 *****************************************************************************/


/*****************************************************************************
 *
 * Description:
 *    Write data to display
 *
 * Params:
 *    [in] data - the data to write.
 *
 ****************************************************************************/
static void
writeToDisp(UNS_16 data)
{
  LCD_DATA_16 = data;
}

/*****************************************************************************
 *
 * Description:
 *    read data from display
 *
 * Return:
 *    Read data
 *
 ****************************************************************************/
//static UNS_16
//readFromDisp(void)
//{
//  volatile UNS_16 value;
//
//  value = LCD_DATA_16;  //dummy data
//  value = LCD_DATA_16;
//
//  return value;
//}

/*****************************************************************************
 *
 * Description:
 *    Write a command
 *
 * Params:
 *    [in] data - the command to send to the display
 *
 ****************************************************************************/
static void
writeToReg(UNS_16 data)
{
  LCD_COMMAND_16 = data;
}

/*****************************************************************************
 *
 * Description:
 *    Write data to a register
 *
 * Params:
 *    [in] addr - register address
 *    [in] data - data to write
 *
 ****************************************************************************/
static void
writeToRegAndDisp(UNS_16 addr, UNS_16 data)
{
  LCD_COMMAND_16 = addr;
  LCD_DATA_16    = data;
}

/*****************************************************************************
 *
 * Description:
 *    Read data from a register
 *
 * Params:
 *    [in] addr - register address
 *
 * Return:
 *    Register value
 *
 ****************************************************************************/
static UNS_16 
readFromReg(UNS_8 addr)
{
  LCD_COMMAND_16 = addr;
  return LCD_DATA_16;
}

/*****************************************************************************
 *
 * Description:
 *    Initialization routine for QVGA LCD v2.x display
 *
 * Return:
 *    TRUE if initialization successful
 *
 ****************************************************************************/
static BOOL_32 
lcdV2Init(void)
{
  if (readFromReg(0) != EA_DISP_QVGA_LCD_V2_ID)
    return FALSE;
  
  /*Display Control Setting*/
  
  writeToRegAndDisp (0x00,0x0001);    //Driver output control register
  delayMs(2);
  writeToRegAndDisp (0x03,0xA2A4);    //LCD Driving waveform control register
  writeToRegAndDisp (0x0C,0x0004);    //BGR=0,
  writeToRegAndDisp (0x0D,0x0308);    //16-bit compare register
  writeToRegAndDisp (0x0E,0x3000);    //display control register 2
  delayMs(10);
  
  writeToRegAndDisp (0x1E,0x00EA);
  
  writeToRegAndDisp (0x01,0x2B3F);    //Frame cycle control register
  writeToRegAndDisp (0x02,0x0600);    //gate scan position register
  writeToRegAndDisp (0x10,0x0000);    //CP/WM 18/16-bit selection register 16bit
  writeToRegAndDisp (0x11,0x6030);    //vertical scroll control
  writeToRegAndDisp (0x07,0x0233);
  writeToRegAndDisp (0x0B,0x0039);
  writeToRegAndDisp (0x0F,0x0000);    //HEA=128,HSA=0
  
  delayMs(5);
  writeToRegAndDisp (0x30,0x0707);    //VEA=160,VSA=0
  writeToRegAndDisp (0x31,0x0204);
  writeToRegAndDisp (0x32,0x0204);
  writeToRegAndDisp (0x33,0x0502);
  writeToRegAndDisp (0x34,0x0507);
  writeToRegAndDisp (0x35,0x0204);
  writeToRegAndDisp (0x36,0x0204);
  writeToRegAndDisp (0x37,0x0502);
  writeToRegAndDisp (0x3A,0x0302);
  writeToRegAndDisp (0x3B,0x0302);
  
  writeToRegAndDisp (0x23,0x0000);    //start oscillation
  writeToRegAndDisp (0x24,0x0000);    //
  writeToRegAndDisp (0x25,0xA000);
  
  writeToRegAndDisp (0x48,0x0000);    //VCOM,VCOMH set
  writeToRegAndDisp (0x49,0x013F);    //REGP=vci
  writeToRegAndDisp (0x4A,0x0000);    //
  writeToRegAndDisp (0x4B,0x0000);    //
  
  writeToRegAndDisp (0x41,0x0000);    //VCOM,VCOMH set
  writeToRegAndDisp (0x42,0x0000);    //VREG1OUT=1.9*REGP
  
  writeToRegAndDisp (0x44,0xEF00);    //
  writeToRegAndDisp (0x45,0x0000);    //fosc/8,
  writeToRegAndDisp (0x46,0x013F);    //VGH=6*VCI1,VGL=-4*VCI1,fosc/8,
  writeToRegAndDisp (0x22,0x0000);	//GON=1 DTE=1 D1-0=11
  
  delayMs(5);
  writeToRegAndDisp (0x44,0xEF00);	//GON=0 DTE=0 D1-0=01
  writeToRegAndDisp (0x45,0x0000);	//GON=1 DTE=0 D1-0=01
  writeToRegAndDisp (0x4E,0x0000);	//GON=1 DTE=0 D1-0=11
  writeToRegAndDisp (0x4F,0x0000);	//GON=1 DTE=1 D1-0=11
  writeToRegAndDisp (0x46,0x013F);	//GON=1 DTE=1 D1-0=11
  writeToRegAndDisp (0x22,0x0000);	//GON=1 DTE=1 D1-0=11

  activeDisplay = EA_DISP_QVGA_LCD_V2;

  return TRUE;  
}

/*****************************************************************************
 *
 * Description:
 *    Initialization routine for QVGA OLED v1.x display
 *
 * Return:
 *    TRUE if initialization successful
 *
 ****************************************************************************/
static BOOL_32 
oledV1Init(void)
{
  UNS_16 result = 0;

  //set 16-bit mode
  writeToReg(0x23);

  //set cpu interface
  writeToReg(0x02);
  writeToDisp(0x0000);

  //set 65k colors, one 16-bit transfer per pixel
  writeToReg(0x03);
  writeToDisp(0x0130);

  writeToReg(0x0f);

  result = LCD_DATA_16;
  if (result != EA_DISP_QVGA_OLED_V1_ID)
  {
	  return FALSE;
  }

  //set standby off
  writeToReg(0x10);
  writeToDisp(0x0000);

  //100ms delay
  delayMs(100);

  //32 ms delay
  delayMs(32);
  
  //set display on
  writeToReg(0x05);
  writeToDisp(0x0001);

  //enable image data transfer
  writeToReg(0x22);

  activeDisplay = EA_DISP_QVGA_OLED_V1;

  return TRUE;  
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Initialize the display
 *
 * Params:
 *    [in] display - Selects which display to initialize. Use any of the 
 *                   EA_DISP_* constants
 *
 * Return:
 *    TRUE if the display got correctly initialized; otherwise FALSE
 *
 ****************************************************************************/
BOOL_32 lcdhw_init(UNS_8 display)
{

  /* QVGA LCD v2.x */

  if (display == EA_DISP_QVGA_LCD_V2)
  {
    return lcdV2Init();
  }

  /* QVGA OLED v1.x */

  else if (display == EA_DISP_QVGA_OLED_V1)
  {
    return oledV1Init();
  }

  return FALSE;
}

/*****************************************************************************
 *
 * Description:
 *    Set position of the pen
 *
 * Params:
 *    [in] x - x position
 *    [in] y - y position
 *
 *
 ****************************************************************************/
void lcdhw_setpos(UNS_16 x, UNS_16 y)
{
  if (activeDisplay == EA_DISP_QVGA_LCD_V2)
  {
    writeToRegAndDisp(0x4e, x & 0xff);
    writeToRegAndDisp(0x4f, y & 0x1ff);
    writeToReg(0x22);
  }

  else if (activeDisplay == EA_DISP_QVGA_OLED_V1)
  {
    writeToReg(0x20);
    writeToDisp(x);
    writeToReg(0x21);
    writeToDisp(y);
    writeToReg(0x22);
  }
}

/*****************************************************************************
 *
 * Description:
 *    Write color data to the display
 *
 * Params:
 *    [in] data - color data
 *
 ****************************************************************************/
void lcdhw_color(UNS_16 data)
{
  writeToDisp(data);
}

